////////////////////////////////////////////////////////////////////////////////
/// @file
/// @brief ネットワークインターフェイス関連のソース
///
/// IPアドレスやMACアドレスの取得に利用されます。
///
/// Copyright (C) 2009 Alpha Systems Inc. All Rights Reserved.
/// @internal
/// @date $Date: $
/// @version $Revision: $
////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <errno.h>
#include <time.h>
#include <sys/ioctl.h>
#include <net/if.h>

int get_ifaddr(char *ifname, char *addr)
{
	int sock;
	struct ifreq ifr;
	char *ipaddr;

	if (!ifname || !addr) {
		return -1;
	}

	// $man netdevice
	sock = socket(AF_INET, SOCK_DGRAM, 0);
	memset(&ifr, 0, sizeof(ifr));
	ifr.ifr_addr.sa_family = AF_INET;
	strncpy(ifr.ifr_name, ifname, IFNAMSIZ - 1);
	if (ioctl(sock, SIOCGIFADDR, &ifr)) {
		close(sock);
		printf("get IP Address failed\n");
		return -1;
	}
	if (ioctl(sock, SIOCGIFFLAGS, &ifr)) {
		close(sock);
		printf("get IP Address failed\n");
		return -1;
	}
	close(sock);

	if (((ifr.ifr_flags & IFF_UP) == 0) || ((ifr.ifr_flags & IFF_RUNNING) == 0)) {
		printf("NIC is not running\n");
		return -1;
	}

	ipaddr = (char *)inet_ntoa(((struct sockaddr_in *)&(ifr.ifr_addr))->sin_addr.s_addr);
	strcpy(addr, ipaddr);

	return 0;
}

int get_macaddr(char *ifname, unsigned char *addr)
{
	int sock;
	struct ifreq ifr;
	int i;

	if (!ifname || !addr) {
		return -1;
	}

	// $man netdevice
	sock = socket(AF_INET, SOCK_DGRAM, 0);
	memset(&ifr, 0, sizeof(ifr));
	ifr.ifr_addr.sa_family = AF_INET;
	strncpy(ifr.ifr_name, ifname, IFNAMSIZ - 1);
	if (ioctl(sock, SIOCGIFHWADDR, &ifr)) {
		close(sock);
		return -1;
	}
	close(sock);

	for (i = 0; i < MACADDR_SIZE; i++) {
		addr[i] = ifr.ifr_hwaddr.sa_data[i];
	}

	return 0;
}

int find_macaddr(int sock, unsigned char *addr) {
	struct sockaddr_in src_addr;
	socklen_t len;
	struct ifconf ifc;
	struct ifreq ifr[8];
	int i;

	memset(&src_addr, 0, sizeof(src_addr));
	len = sizeof(src_addr);
	if (getsockname(sock, (struct sockaddr *)&src_addr, &len)) {
		return -1;
	}

	memset(&ifc, 0, sizeof(ifc));
	memset(&ifr, 0, sizeof(ifr));
	ifc.ifc_len = sizeof(ifr);
	ifc.ifc_req = ifr;
	if (ioctl(sock, SIOCGIFCONF, &ifc)) {
		return -1;
	}

	for (i = 0; i < ifc.ifc_len / sizeof(struct ifreq); i++) {
		if (*(in_addr_t *)&(src_addr.sin_addr) == *(in_addr_t *)&((struct sockaddr_in *)&(ifc.ifc_req[i].ifr_addr))->sin_addr) {
			return get_macaddr(ifc.ifc_req[i].ifr_name, addr);
		}
	}

	return -1;
}
